package org.fouram.service;

import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.fouram.constants.WebConstants;
import org.fouram.core.base.model.PageData;
import org.fouram.core.base.service.BaseService;
import org.fouram.core.base.service.RequestService;
import org.fouram.core.plugin.weixin.cp.util.WXCpMessageUtil;
import org.fouram.core.plugin.weixin.cp.util.WXCpUserUtil;
import org.fouram.core.util.Base64Util;
import org.fouram.core.util.ConfConfig;
import org.fouram.core.util.DateUtil;
import org.fouram.core.util.JSONUtil;
import org.fouram.core.util.JsonNodeUtil;
import org.fouram.core.util.LoggerUtil;
import org.fouram.core.util.Tools;
import org.fouram.dto.input.GetIngoreListDTO;
import org.fouram.dto.input.GetToBeConfirmedListDTO;
import org.fouram.dto.output.GetIngoreListOutput;
import org.fouram.dto.output.GetToBeConfirmedListOutput;
import org.fouram.entity.SapMessageUser;
import org.fouram.entity.SapOrg;
import org.fouram.entity.SapUser;
import org.fouram.entity.WxUser;
import org.fouram.enums.DelFlagEnum;
import org.fouram.enums.SapUserResultEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.fasterxml.jackson.databind.JsonNode;
import com.google.common.collect.Lists;

import jodd.util.StringUtil;
import me.chanjar.weixin.cp.bean.WxCpUser;

@Service
@Transactional
public class SapUserService extends BaseService {

	@Autowired
	private SapOrgService sapOrgService;
	@Autowired
	private WxUserService wxUserService;
	@Autowired
	private RequestService requestService;
	@Autowired
	private SapEmpJobService sapEmpJobService;
	@Autowired
	private SapOprLogService sapOprLogService;
	@Autowired
	private SapPersonalService sapPersonalService;
	@Autowired
	private SapBlacklistService sapBlacklistService;
	@Autowired
	private SapEmpEmploymentService sapEmpEmploymentService;
	@Autowired
	private SapEmpEmploymentTerminationService sapEmpEmploymentTerminationService;

	public void saveOrUpdateSapUsers() throws Exception {
		HttpHeaders headers = getHttpHeaders();
		String userUrl = "https://api15.sapsf.cn/odata/v2/User?$filter=status in 'active'&$format=json&$expand=personKeyNav";
		// get
		JsonNode node = requestService.getForObject(userUrl, headers);
		List<SapUser> sapUsers = Lists.newArrayList();
		StringBuffer name = null;
		if (node != null) {
			Iterator<JsonNode> iterator = node.get("d").get("results").elements();
			while (iterator.hasNext()) {
				JsonNode userNode = iterator.next();
				String userId = JsonNodeUtil.getValue(userNode, "userId");
				// 去除非员工账号和测试数据
				if (!userId.startsWith("Test")) {
					String username = JsonNodeUtil.getValue(userNode, "username");
					name = new StringBuffer().append(JsonNodeUtil.getValue(userNode, "lastName"))
							.append(JsonNodeUtil.getValue(userNode, "firstName"));
					String mobile = JsonNodeUtil.getValue(userNode, "cellPhone");
					String businessPhone = JsonNodeUtil.getValue(userNode, "businessPhone");
					String email = JsonNodeUtil.getValue(userNode, "email");
					String personId = JsonNodeUtil.getValue(userNode, "personKeyNav", "personIdExternal");
					SapUser sapUser = SapUser.builder().userId(userId).personId(personId).username(username)
							.name(name.toString()).mobile(mobile).formatMobile(mobile).businessPhone(businessPhone)
							.formatBusinessPhone(formatBusinessPhone(businessPhone)).email(email).formatEmail(email)
							.isPrimary(userId.equals(personId)).build();
					sapUsers.add(sapUser);
				}
			}
		}
		// 获取用户对应机构
		sapOrgService.setOrg(sapUsers);
		LoggerUtil.info("saveWxDeparts->setOrg:" + DateUtil.getTime());
		// 获取用户对应性别
		sapPersonalService.setGender(sapUsers);
		LoggerUtil.info("saveWxDeparts->setGender:" + DateUtil.getTime());
		// 获取用户对应职务
		sapEmpJobService.setPosition(sapUsers);
		LoggerUtil.info("saveWxDeparts->setPosition:" + DateUtil.getTime());
		// EmpEmployment->startDate
		sapEmpEmploymentService.setStartDate(sapUsers);
		LoggerUtil.info("saveWxDeparts->setStartDate:" + DateUtil.getTime());
		// EmpEmploymentTermination->endDate
		sapEmpEmploymentTerminationService.setEndDate(sapUsers);
		LoggerUtil.info("saveWxDeparts->setEndDate:" + DateUtil.getTime());
		// 更新用户
		SapUser sapUserDb = null;
		SapUser sapUser = null;
		for (int i = 0; i < sapUsers.size(); i++) {
			sapUser = sapUsers.get(i);
			sapUser.setSortNumber(i + 1);
			sapUserDb = this.selectByUserId(sapUser.getUserId());
			if (sapUserDb == null) {
				this.saveSapUser(sapUser);
			} else {
				// 如果是忽略INGORE，保留INGORE状态
				if (SapUserResultEnum.INGORE.getCode().equals(sapUserDb.getResult())) {
					sapUser.setResult(SapUserResultEnum.INGORE.getCode());
				} else {
					sapUser.setResult(SapUserResultEnum.NORMAL.getCode());
				}
				this.updateSapUser(sapUser);
			}
		}
	}

	public String getCheckResultByWx(SapUser sapUser) throws Exception {
		if (Tools.isEmpty(sapUser.getPersonId()) || Tools.isEmpty(sapUser.getFirstOrgName())
				|| WebConstants.INGORE_PERSONIDS.contains("," + sapUser.getPersonId() + ",")) {
			return SapUserResultEnum.GARBAGE.getCode();
		} else if (sapBlacklistService.findByUserId(sapUser.getPersonId()) != null) {
			return SapUserResultEnum.BLACKLIST.getCode();
		} else {
			List<SapUser> sapUsers = selectListByPersonId(sapUser.getPersonId());
			WxUser wxUser = wxUserService.selectById(sapUser.getPersonId(), sapUser.getEmail(),
					DelFlagEnum.ENABLE.getCode());
			sapUser.setSapPosition(getFullPosition(sapUsers));
			sapUser.setSapFullDepart(getFullOrgName(sapUsers));
			if (wxUser == null) {
				return SapUserResultEnum.ENTRY.getCode();
			} else {
				sapUser.setWxPosition(wxUser.getPosition());
				sapUser.setWxFullDepart(wxUserService.getFullDepartName(sapUser.getPersonId()));
				if (!sapUser.getSapPosition().equals(sapUser.getWxPosition())
						|| !sapUser.getSapFullDepart().equals(sapUser.getWxFullDepart())) {
					return SapUserResultEnum.SENSITIVE.getCode();
				} else { // 因为上面已经验证过敏感数据，这次判断其他数据是否修改过 就可以
					// name+mobile+email+telephone
					if ((StringUtil.equals(wxUser.getEmail(), sapUser.getEmail())
							|| StringUtil.equals(wxUser.getUserId(), sapUser.getEmail()))
							&& StringUtil.equals(wxUser.getMobile(), sapUser.getMobile())
							&& StringUtil.equals(wxUser.getName(), sapUser.getName())
							&& StringUtil.equals(wxUser.getTelephone(), sapUser.getBusinessPhone())) {
						return SapUserResultEnum.SUCCESS.getCode();
					}
				}
			}
		}
		return null;
	}

	/**
	 * 更新异常情况，敏感数据、入职
	 * 
	 * @throws Exception
	 */
	public void updateAbnormalResult() throws Exception {
		// 查询NORMAL有效数据，判断异常，入职，没变化
		List<SapUser> normalList = selectPrimaryListByResult(SapUserResultEnum.NORMAL.getCode());
		for (SapUser sapUser : normalList) {
			String checkResult = getCheckResultByWx(sapUser);
			if (StringUtils.isNotBlank(checkResult)) {
				updateResultByUserId(checkResult, sapUser.getUserId());
			}
			// 更新sapPosition、wxPosition、sapFullDepart、wxFullDepart
			updateFullDepartAndPosition(sapUser);
		}
		// 查询UNABLE数据，判断已不存在或离职
		List<SapUser> unableList = selectPrimaryListByDelFlag(DelFlagEnum.UNABLE.getCode());
		for (SapUser sapUser : unableList) {
			WxUser wxUser = wxUserService.selectById(sapUser.getPersonId(), sapUser.getEmail(),
					DelFlagEnum.ENABLE.getCode());
			if (wxUser != null) {
				updateResultByUserId(SapUserResultEnum.LEAVE.getCode(), sapUser.getUserId());
			} else {
				updateResultByUserId(SapUserResultEnum.NON_EXISTENT.getCode(), sapUser.getUserId());
			}
		}
	}

	/**
	 * 根据personId查询多个用户完整的机构名称信息
	 * 
	 * @param sapUsers
	 * @return
	 * @throws Exception
	 */
	public String getFullOrgName(List<SapUser> sapUsers) throws Exception {
		StringBuffer sb = new StringBuffer();
		for (SapUser user : sapUsers) {
			if (sb.indexOf(user.getFullOrgName()) < 0) {
				sb.append(user.getFullOrgName()).append(";");
			}
		}
		if (sb.length() > 0) {
			return sb.substring(0, sb.length() - 1);
		}
		return "";
	}

	/**
	 * 拼接多用户的职务
	 * 
	 * @param sapUsers
	 * @return
	 * @throws Exception
	 */
	public String getFullPosition(List<SapUser> sapUsers) throws Exception {
		StringBuffer sb = new StringBuffer();
		for (SapUser user : sapUsers) {
			if (sb.indexOf(user.getFormatPosition()) < 0) {
				sb.append(user.getFormatPosition()).append("、");
			}
		}
		if (sb.length() > 0) {
			return sb.substring(0, sb.length() - 1);
		}
		return "";
	}

	public void saveSapUser(SapUser entity) throws Exception {
		entity.setCreateDate(new Date());
		entity.setResult(SapUserResultEnum.NORMAL.getCode());
		entity.setDelFlag(DelFlagEnum.ENABLE.getCode());
		save("SapUserMapper.saveOne", entity);
	}

	public void updateSapUser(SapUser entity) throws Exception {
		entity.setDelFlag(DelFlagEnum.ENABLE.getCode());
		update("SapUserMapper.updateOne", entity);
	}

	@Transactional
	public void updateSync(List<String> userIds) throws Exception {
		SapUser sapUser = null;
		for (String userId : userIds) {
			sapUser = this.selectByUserId(userId);
			if (SapUserResultEnum.LEAVE.getCode().equals(sapUser.getResult())) {
				this.deleteQuitUserByUserId(userId, null);
			} else {
				this.updateSyncWxResult(userId);
			}
		}
	}

	public void updateSyncWxResult(String personId) throws Exception {
		String result = createOrUpdateWXUser(personId);
		updateResultByUserId(result, personId);
	}

	private String createOrUpdateWXUser(String personId) throws Exception {
		List<SapUser> sapUsers = selectListByPersonId(personId);
		SapUser primarySapUser = sapUsers.get(0);
		String formatMobile = primarySapUser.getFormatMobile();
		// 测试企业微信，手机号码脱敏
		// fouram wxCp.corpId = ww51602aeb8dab7a95
		// prod wxCp.corpId = ww911e29458d3a46fd
		if (!ConfConfig.getConfigString("wxCp.corpId").equals("ww911e29458d3a46fd")) {
			if (StringUtils.isNotBlank(formatMobile) && formatMobile.length() == 11) {
				formatMobile = formatMobile.substring(0, 8) + "999";
			} else {
				formatMobile = "";
			}
		}

		Long[] firstWxDepartIdList = new Long[sapUsers.size()];
		Long[] secondWxDepartIdList = new Long[sapUsers.size()];
		Long[] thirdWxDepartIdList = new Long[sapUsers.size()];
		Long[] fourthWxDepartIdList = new Long[sapUsers.size()];
		for (int i = 0; i < sapUsers.size(); i++) {
			SapUser sapUser = sapUsers.get(i);
			Map<String, Long> map = sapOrgService.updateWxDepartId(sapUser);
			firstWxDepartIdList[i] = map.get(sapUser.getFirstOrgId());
			secondWxDepartIdList[i] = map.get(sapUser.getSecondOrgId());
			thirdWxDepartIdList[i] = map.get(sapUser.getThirdOrgId());
			fourthWxDepartIdList[i] = map.get(sapUser.getFourthOrgId());
		}
		return WXCpUserUtil.createOrUpdate(personId, primarySapUser.getName(), firstWxDepartIdList,
				secondWxDepartIdList, thirdWxDepartIdList, fourthWxDepartIdList, primarySapUser.getSapPosition(),
				formatMobile, primarySapUser.getGender(), primarySapUser.getRealEmail(),
				primarySapUser.getFormatBusinessPhone(), primarySapUser.getFirstOrgName());
	}

	public SapUser selectByUserId(String userId) throws Exception {
		return (SapUser) findObject("SapUserMapper.selectByUserId", userId);
	}

	@SuppressWarnings("unchecked")
	public List<SapUser> selectListByPersonId(String personId) throws Exception {
		return (List<SapUser>) findList("SapUserMapper.selectListByPersonId", personId);
	}

	public void updateAllDelFlag(Integer delFlag) throws Exception {
		update("SapUserMapper.updateAllDelFlag", delFlag);
	}

	public void updateResultByDelFlag(SapUserResultEnum sapUserResultEnum, DelFlagEnum delFlagEnum) throws Exception {
		SapUser entity = SapUser.builder().result(sapUserResultEnum.getCode()).delFlag(delFlagEnum.getCode()).build();
		update("SapUserMapper.updateResultByDelFlag", entity);
	}

	public void updateDelFlag(String userId, DelFlagEnum delFlagEnum) throws Exception {
		SapUser entity = SapUser.builder().userId(userId).delFlag(delFlagEnum.getCode()).build();
		update("SapUserMapper.updateDelFlag", entity);
	}

	public static void main(String[] args) {
		StringBuilder stringBuilder = new StringBuilder();
		String sapApiUser = "API01@cmccoltd:cmccoltd123";
		String authorizationStr = stringBuilder.append("Basic ").append(Base64Util.encodeString(sapApiUser)).toString();
		System.out.println(authorizationStr);
	}

	@SuppressWarnings("unchecked")
	public List<SapUser> selectPrimaryListByName(String name) throws Exception {
		return (List<SapUser>) findList("SapUserMapper.selectPrimaryListByName", name);
	}

	@SuppressWarnings("unchecked")
	public List<SapUser> selectPrimaryListByResult(String result) throws Exception {
		return (List<SapUser>) findList("SapUserMapper.selectPrimaryListByResult", result);
	}

	@SuppressWarnings("unchecked")
	public List<SapUser> selectPrimaryListByDelFlag(Integer delFlag) throws Exception {
		return (List<SapUser>) findList("SapUserMapper.selectPrimaryListByDelFlag", delFlag);
	}

	@SuppressWarnings("unchecked")
	public List<SapUser> selectPrimaryListByDepartIdAndLevel(String departId) throws Exception {
		SapOrg sapOrg = sapOrgService.selectById(departId);
		if (StringUtils.isBlank(departId) || WebConstants.TOP_DEPART.toString().equals(departId)) {
			departId = WebConstants.TOP_DEPART.toString();
			sapOrg = SapOrg.builder().id(WebConstants.TOP_DEPART.toString()).sapName(WebConstants.TOP_DEPART_NAME)
					.level("1").build();
		}
		PageData pd = new PageData();
		pd.put("departId", departId);
		pd.put("level", sapOrg.getLevel());
		return (List<SapUser>) findList("SapUserMapper.selectPrimaryListByDepartIdAndLevel", pd);
	}

	public SapUser toInfoDetail(SapUser sapUser) throws Exception {
		if (sapUser != null) {
			List<SapUser> users = selectListByPersonId(sapUser.getPersonId());
			List<List<SapOrg>> departList = Lists.newArrayList();
			for (int i = 0; i < users.size(); i++) {
				SapUser user = users.get(i);
				List<SapOrg> orgs = Lists.newArrayList();
				if (Tools.notEmpty(user.getFirstOrgId())) {
					orgs.add(sapOrgService.selectById(user.getFirstOrgId()));
				}
				if (Tools.notEmpty(user.getSecondOrgId())) {
					orgs.add(sapOrgService.selectById(user.getSecondOrgId()));
				}
				if (Tools.notEmpty(user.getThirdOrgId())) {
					orgs.add(sapOrgService.selectById(user.getThirdOrgId()));
				}
				if (Tools.notEmpty(user.getFourthOrgId())) {
					orgs.add(sapOrgService.selectById(user.getFourthOrgId()));
				}
				departList.add(orgs);
			}
			sapUser.setDepartList(departList);
		}
		return sapUser;
	}

	public List<SapUser> toInfoDetails(List<SapUser> sapUsers) throws Exception {
		for (SapUser sapUser : sapUsers) {
			toInfoDetail(sapUser);
		}
		return sapUsers;
	}

	public SapUser getDetailById(String userId) throws Exception {
		SapUser sapUser = selectByUserId(userId);
		return toInfoDetail(sapUser);
	}

	// 更新用户头像
	public void updateAvatars() throws Exception {
		List<SapUser> successUsers = selectPrimaryListByResult(SapUserResultEnum.SUCCESS.getCode());
		for (SapUser sapUser : successUsers) {
			WxCpUser user = WXCpUserUtil.getByIdOrEmail(sapUser.getPersonId(), sapUser.getEmail());
			if (user != null && StringUtils.isNotBlank(user.getAvatar())) {
				sapUser.setAvatar(user.getAvatar());
				update("SapUserMapper.updateAvatar", sapUser);
			}
		}
	}

	public void updateFullDepartAndPosition(SapUser sapUser) throws Exception {
		update("SapUserMapper.updateFullDepartAndPosition", sapUser);
	}

	public void updateResultByUserId(String result, String userId) throws Exception {
		SapUser sapUser = SapUser.builder().userId(userId).result(result).build();
		if (SapUserResultEnum.INGORE.getCode().equals(result)) {
			sapUser.setIngoreDate(new Date());
		}
		update("SapUserMapper.updateResultByUserId", sapUser);
	}

	public void deleteQuitUserByUserId(String userId, String oprUserId) throws Exception {
		SapUser sapUser = selectByUserId(userId);
		String personId = sapUser.getPersonId();
		List<SapUser> sapUsers = selectListByPersonId(personId);
		for (SapUser user : sapUsers) {
			updateDelFlag(user.getUserId(), DelFlagEnum.DELETED);
		}
		WXCpUserUtil.delete(personId, sapUser.getEmail());
		sapOprLogService.saveOne(JSONUtil.toString(sapUsers), WebConstants.OPRTYPE_DELETE, oprUserId);
	}

	@SuppressWarnings("unchecked")
	public GetToBeConfirmedListOutput selectToBeConfirmedPrimaryList(GetToBeConfirmedListDTO dto) throws Exception {
		PageData pd = new PageData();
		pd.put("name", dto.getName());
		pd.put("type", dto.getType());
		if (dto.getPageNumber() != null && dto.getPageSize() != null) {
			pd.put("pageCurrent", String.valueOf(dto.getPageSize() * (dto.getPageNumber() - 1)));
			pd.put("pageSize", String.valueOf(dto.getPageSize()));
		}
		List<SapUser> data = (List<SapUser>) findList("SapUserMapper.selectToBeConfirmedPrimaryList", pd);
		Long total = (Long) findObject("SapUserMapper.selectToBeConfirmedPrimaryTotal", pd);
		return GetToBeConfirmedListOutput.builder().data(toInfoDetails(data)).total(total).build();
	}

	@Transactional
	public void updateIngore(List<String> personIds) throws Exception {
		for (String personId : personIds) {
			updateResultByUserId(SapUserResultEnum.INGORE.getCode(), personId);
		}
	}

	public void updateCancelIngore(String personId) throws Exception {
		SapUser sapUser = selectByUserId(personId);
		String result = getCheckResultByWx(sapUser);
		if (StringUtils.isBlank(result)) {
			result = SapUserResultEnum.NORMAL.getCode();
		}
		updateResultByUserId(result, personId);
	}

	public void updateAllNormalSyncWxResult() throws Exception {
		List<SapUser> normalSapUsers = this.selectPrimaryListByResult(SapUserResultEnum.NORMAL.getCode());
		for (SapUser sapUser : normalSapUsers) {
			// 同步提示报错，让用户所有的都执行完
			try {
				this.updateSyncWxResult(sapUser.getPersonId());
			} catch (Exception e) {
				LoggerUtil.error(e);
			}
		}
	}

	@SuppressWarnings("unchecked")
	public GetIngoreListOutput selectIngorePrimaryList(GetIngoreListDTO dto) throws Exception {
		PageData pd = new PageData();
		pd.put("name", dto.getName());
		pd.put("result", SapUserResultEnum.INGORE.getCode());
		if (dto.getPageNumber() != null && dto.getPageSize() != null) {
			pd.put("pageCurrent", String.valueOf(dto.getPageSize() * (dto.getPageNumber() - 1)));
			pd.put("pageSize", String.valueOf(dto.getPageSize()));
		}
		List<SapUser> data = (List<SapUser>) findList("SapUserMapper.selectIngorePrimaryList", pd);
		Long total = (Long) findObject("SapUserMapper.selectIngorePrimaryTotal", pd);
		return GetIngoreListOutput.builder().data(toInfoDetails(data)).total(total).build();
	}

	@SuppressWarnings("unchecked")
	public List<SapUser> selectNotSendEntryMessageList() throws Exception {
		return (List<SapUser>) findList("SapUserMapper.selectNotSendEntryMessageList", null);
	}

	@Transactional
	public void sendEntryMessage(String messageContent, SapUser sapUser, List<SapMessageUser> sapMessageUsers)
			throws Exception {
		for (SapMessageUser sapMessageUser : sapMessageUsers) {
			WXCpMessageUtil.send(messageContent, sapMessageUser.getUserId());
		}
		updateSendEntryMessageFlag(sapUser.getUserId(), true);
	}

	private void updateSendEntryMessageFlag(String userId, boolean sendEntryMessageFlag) throws Exception {
		SapUser sapUser = SapUser.builder().userId(userId).sendEntryMessageFlag(sendEntryMessageFlag).build();
		update("SapUserMapper.updateSendEntryMessageFlag", sapUser);
	}
}
